// **************************************************
// This file is part of the ROPI Library.
// Written by Marc Goerigk
// Institut für Numerische und Angewandte Mathematik
// Georg-August Universität Göttingen
// **************************************************

#include<iostream>
#include<fstream>

#include "lp.h"
#include "solver.h"
#include "error.h"

#include<vector>
#include<cstdio>

using namespace std;
using namespace ROPI;

int main(int argc, char* argv[])
{
  cout<<"**********\nROPI check.\nSolves a MIP problem with all available solvers.\n";
  cout<<"Usage: ./check FILE.mps\n**********\n";
  if (argc != 2)
    return 0;
  
  try
  {
    LP lp;
    lp.read_mpsfile(argv[1]);
    
    list<SOLVERTYPE> types = Solver::get_available();
      
    SolverOptions options;
    options.verbose = false;
    
    for (list<SOLVERTYPE>::iterator it=types.begin(); it!=types.end(); ++it)
    {
      switch (*it)
      {
		case SOL_GUROBI: cout<<argv[1]<<";GUROBI;"; break;
		case SOL_CPLEX: cout<<argv[1]<<";CPLEX;"; break;
		case SOL_XPRESS: cout<<argv[1]<<";XPRESS;"; break;
		default: cout<<argv[1]<<";UNKNOWN;";
      }
      
      Solver solver;
      solver.init(&lp, *it);
      solver.set_options(options);
      
      clock_t begin = clock();
      solver.solve();
      clock_t end = clock();
      
      if (solver.get_status() == SOL_OPTIMAL)
		cout<<solver.get_objective()<<";"<<1.0* (end - begin) / CLOCKS_PER_SEC<<"\n"<<flush;
      else
		cout<<"NOT OPTIMAL;"<<1.0* (end - begin) / CLOCKS_PER_SEC<<"\n"<<flush;
    }
  }
  catch (Error &err)
  {
    cout<<"An error occurred:\n"<<err.get_mesg()<<"\n";
  }
  catch (exception &err)
  {
    cout<<"An error occurred:\n"<<err.what()<<"\n";
  }
  catch (...)
  {
    cout<<"An unknown error occurred.\n";
  }
  
  
  return 0; 
}
