// **************************************************
// This file is part of the GOROBO Library.
// Written by Marc Goerigk
// Institut für Numerische und Angewandte Mathematik
// Georg-August Universität Göttingen
// **************************************************

#include<iostream>
#include<fstream>

#include "lp.h"
#include "solver.h"
#include "error.h"
#include "robustness.h"

#include<vector>
#include<cstdio>
#include<cstdlib>
#include<ctime>

using namespace std;
using namespace ROPI;

int main(int argc, char* argv[])
{
  cout<<"**********\nROPI robustness check.\nSolves a robust MIP problem.\n";
  cout<<"Usage: ./robust_check FILE.mps\n**********\n";
  if (argc != 2)
    return 0;
  
  try
  {
    srand(time(0));

    LP lp;
    lp.read_mpsfile(argv[1]);
    lp.write_plain(cout);
    
    FiniteUncertainty unc;
    list<Con>* lpcons = lp.get_cons();
    for (int i=0; i<2; ++i)
    {
      list<Con>::iterator it = lpcons->begin();
      advance(it,rand()%(lpcons->size()));
      list<pair<int,double> >::iterator it2=it->coeffs.begin();
      advance(it2,rand()%(it->coeffs.size()));
      FiniteScenario scen;
      scen.coeffs[make_pair<Con,int>(*it,it2->first)] = it2->second*(1 + (rand()%100)/400.0 - 1.0/8 );
      unc.scenarios.push_back(scen);  
    }
    
    Robustness strict(&lp,ROB_STRICT);
    strict.set_uncertainty(unc);
    LP rc = strict.generate_robust();
    rc.write_plain(cout);

    Solver sol;
    sol.init(&rc, SOL_GUROBI);
    sol.solve();
    
    if (sol.get_status() == SOL_OPTIMAL)
	   sol.write_solution(cout,-1);
    else
       cout<<"Could not solve robust model successfully..\n";
    
    
  }
  catch (Error &err)
  {
    cout<<"An error occurred:\n"<<err.get_mesg()<<"\n";
  }
  catch (exception &err)
  {
    cout<<"An error occurred:\n"<<err.what()<<"\n";
  }
  catch (...)
  {
    cout<<"An unknown error occurred.\n";
  }
  
  
  return 0; 
}
