#!/bin/sh

set -e

GUROBIPATH=${GUROBI_HOME}
GUROBIVERSION=`(ls ${GUROBI_HOME}/lib/libgurobi*.so | rev | cut -d"i" -f1 | rev | cut -d"." -f1)`
CPLEXPATH="/usr/local/cplex/"
CONCERTPATH="/usr/local/concert"
CPLEXARCH="x86-64_debian4.0_4.1/static_pic"
XPRESSPATH=${XPRESSDIR}

INC=" -Iinclude/"
# FLAGS=" -O2 -Wall -Weffc++"
FLAGS=" -O2 -Wall"
SRC="src"
EXAMPLE="example"
BUILD="build"
OBJS="${BUILD}/lp.o ${BUILD}/lpimpl.o ${BUILD}/solver.o ${BUILD}/robustness.o ${BUILD}/rimpl_recfeas.o ${BUILD}/rimpl_strict.o ${BUILD}/rimpl_recopt.o ${BUILD}/rimpl_light.o"
LIB=""

echo "Building lp interface."
echo "g++ -c ${SRC}/lp.cpp -o ${BUILD}/lp.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/lp.cpp -o ${BUILD}/lp.o ${INC} ${FLAGS} -I${SRC}

echo "g++ -c ${SRC}/lpimpl.cpp -o ${BUILD}/lpimpl.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/lpimpl.cpp -o ${BUILD}/lpimpl.o ${INC} ${FLAGS} -I${SRC}

echo ""
echo "Building solver interface."

echo "g++ -c ${SRC}/solver.cpp -o ${BUILD}/solver.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/solver.cpp -o ${BUILD}/solver.o ${INC} ${FLAGS} -I${SRC}

echo ""
echo "Building robustness models."

echo "g++ -c ${SRC}/robustness.cpp -o ${BUILD}/robustness.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/robustness.cpp -o ${BUILD}/robustness.o ${INC} ${FLAGS} -I${SRC}
echo "g++ -c ${SRC}/rimpl_recfeas.cpp -o ${BUILD}/rimpl_recfeas.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/rimpl_recfeas.cpp -o ${BUILD}/rimpl_recfeas.o ${INC} ${FLAGS} -I${SRC}
echo "g++ -c ${SRC}/rimpl_strict.cpp -o ${BUILD}/rimpl_strict.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/rimpl_strict.cpp -o ${BUILD}/rimpl_strict.o ${INC} ${FLAGS} -I${SRC}
echo "g++ -c ${SRC}/rimpl_recopt.cpp -o ${BUILD}/rimpl_recopt.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/rimpl_recopt.cpp -o ${BUILD}/rimpl_recopt.o ${INC} ${FLAGS} -I${SRC}
echo "g++ -c ${SRC}/rimpl_light.cpp -o ${BUILD}/rimpl_light.o ${INC} ${FLAGS} -I${SRC}"
g++ -c ${SRC}/rimpl_light.cpp -o ${BUILD}/rimpl_light.o ${INC} ${FLAGS} -I${SRC}

echo ""

if test -z "${GUROBIPATH}";then
  echo "No Gurobi installation found."
  OBJS="${OBJS} ${BUILD}/solver_gurobi_dummy.o"
  echo "g++ -c ${SRC}/solver_gurobi_dummy.cpp -o ${BUILD}/solver_gurobi_dummy.o ${INC} ${FLAGS} -I${SRC}"
  g++ -c ${SRC}/solver_gurobi_dummy.cpp -o ${BUILD}/solver_gurobi_dummy.o ${INC} ${FLAGS} -I${SRC}
  echo ""
else
  echo "Gurobi found, version: ${GUROBIVERSION}"
  OBJS="${OBJS} ${BUILD}/solver_gurobi.o"
  LIB="${LIB} -L${GUROBIPATH}/lib -lgurobi_c++ -lgurobi${GUROBIVERSION}"
  INC="${INC} -I${GUROBIPATH}/include"
  echo "g++ -c ${SRC}/solver_gurobi.cpp -o ${BUILD}/solver_gurobi.o ${INC} ${FLAGS} -I${SRC}"
  g++ -c ${SRC}/solver_gurobi.cpp -o ${BUILD}/solver_gurobi.o ${INC} ${FLAGS} -I${SRC}
  echo ""
fi

if test -z "${CPLEXPATH}";then
  echo "No Cplex installation found."
  OBJS="${OBJS} ${BUILD}/solver_cplex_dummy.o"
  echo "g++ -c ${SRC}/solver_cplex_dummy.cpp -o ${BUILD}/solver_cplex_dummy.o ${INC} ${FLAGS} -I${SRC}"
  g++ -c ${SRC}/solver_cplex_dummy.cpp -o ${BUILD}/solver_cplex_dummy.o ${INC} ${FLAGS} -I${SRC}
  echo ""
else
  echo "Cplex found."
  if test -z "${CONCERTPATH}" || test -z "${CPLEXARCH}"; then
    echo "No Concert found. Cannot use Cplex."
    OBJS="${OBJS} ${BUILD}/solver_cplex_dummy.o"
    echo "g++ -c ${SRC}/solver_cplex_dummy.cpp -o ${BUILD}/solver_cplex_dummy.o ${INC} ${FLAGS} -I${SRC}"
    g++ -c ${SRC}/solver_cplex_dummy.cpp -o ${BUILD}/solver_cplex_dummy.o ${INC} ${FLAGS} -I${SRC}
    echo ""
  else
    INC="${INC} -I${CPLEXPATH}/include -I${CONCERTPATH}/include"
    OBJS="${OBJS} ${BUILD}/solver_cplex.o"
    LIB="${LIB} -lilocplex -lcplex  -L${CPLEXPATH}/lib/${CPLEXARCH} -L${CONCERTPATH}/lib/${CPLEXARCH} -lconcert"
    echo "g++ -c ${SRC}/solver_cplex.cpp -o ${BUILD}/solver_cplex.o ${INC} ${FLAGS} -DIL_STD -I${SRC}"
    g++ -c ${SRC}/solver_cplex.cpp -o ${BUILD}/solver_cplex.o ${INC} ${FLAGS} -DIL_STD -I${SRC}
    echo ""
  fi
fi

if test -z "${XPRESSPATH}";then
  echo "No Xpress found."
  OBJS="${OBJS} ${BUILD}/solver_xpress_dummy.o"
  echo "g++ -c ${SRC}/solver_xpress_dummy.cpp -o ${BUILD}/solver_xpress_dummy.o ${INC} ${FLAGS} -I${SRC}"
  g++ -c ${SRC}/solver_xpress_dummy.cpp -o ${BUILD}/solver_xpress_dummy.o ${INC} ${FLAGS} -I${SRC}
  echo ""
else
  echo "Xpress found."
  INC="${INC} -I${XPRESSPATH}/include"
  OBJS="${OBJS} ${BUILD}/solver_xpress.o"
  LIB="${LIB} -lxprb -L${XPRESSPATH}/lib"
  echo "g++ -c ${SRC}/solver_xpress.cpp -o ${BUILD}/solver_xpress.o ${INC} ${FLAGS} -I${SRC}"
  g++ -c ${SRC}/solver_xpress.cpp -o ${BUILD}/solver_xpress.o ${INC} ${FLAGS} -I${SRC}
  echo ""
fi


echo "Linking library."
echo "ar rcs lib/libropi.a ${OBJS}"
ar rcs lib/libropi.a ${OBJS}
echo ""

echo "Building examples."
echo "g++ ${EXAMPLE}/check.cpp -o ${BUILD}/check ${INC} ${FLAGS} -Llib -lropi ${LIB} -lpthread -lm -m64"
g++ ${EXAMPLE}/check.cpp -o ${BUILD}/check ${INC} ${FLAGS} -Llib -lropi ${LIB} -lpthread -lm -m64
echo ""
echo "g++ ${EXAMPLE}/robust_check.cpp -o ${BUILD}/robust_check ${INC} ${FLAGS} -Llib -lropi ${LIB} -lpthread -lm -m64"
g++ ${EXAMPLE}/robust_check.cpp -o ${BUILD}/robust_check ${INC} ${FLAGS} -Llib -lropi ${LIB} -lpthread -lm -m64
echo ""

echo "Cleaning up."
echo "rm ${BUILD}/*.o"
rm ${BUILD}/*.o
echo ""

echo "Building done!"
