// **************************************************
// This file is part of the ROPI Library.
// Written by Marc Goerigk
// Institut für Numerische und Angewandte Mathematik
// Georg-August Universität Göttingen
// **************************************************

#ifndef _LPIMPL_H_
#define _LPIMPL_H_

#include "lp.h"
#include<vector>
#include<string>

namespace ROPI
{
  class LPimpl
  {
    public:
      LPimpl() : name("DEFAULTNAME"), obj_sen(MIN), num_vars(0), num_cons(0), lptype(LPTYPE_LP) {};
      ~LPimpl() {};
      void add_var(Var _var);
      void add_con(Con _con);
      
      void set_name(std::string _name);
      void set_sense(OBJ_SENSE _sen);
      
      std::string get_name();
      OBJ_SENSE get_sense();
      
      std::list<Var>* get_vars();
      std::list<Con>* get_cons();
      int get_num_vars();
      int get_num_cons();
      LPTYPE get_lptype();
      
      void read_mpsfile(std::string file);
      void write_plain(std::ostream &out);
      
    private:
      std::string name;
      OBJ_SENSE obj_sen;
      int num_vars;
      int num_cons;
      std::list<Var> vars;
      std::list<Con> cons;
      std::list<Var*> obj;
      std::map<int,Var*> idvarmap;
      LPTYPE lptype;
      std::vector<std::string> tokenize(std::string line);
      void read_mpsfile_fixed(std::string file);
      void read_mpsfile_free(std::string file);
      
      enum LP_READMODE
      {
	LP_OBJECTIVESENSE = 1,
	LP_OBJECTIVE = 2,
	LP_CONSTRAINTS = 3,
	LP_BOUNDS = 4,
	LP_BINARIES = 5
      };
      
      enum MPS_READMODE
      {
	MPS_NAME = 1,
	MPS_ROWS = 2,
	MPS_COLUMNS = 3,
	MPS_RHS = 4,
	MPS_RANGES = 5,
	MPS_BOUNDS = 6,
	MPS_NONE = 7
      };
      
  };
  
}

#endif
