// **************************************************
// This file is part of the ROPI Library.
// Written by Marc Goerigk
// Institut für Numerische und Angewandte Mathematik
// Georg-August Universität Göttingen
// **************************************************

#include "rimpl_strict.h"
#include "error.h"
#include <sstream>

using namespace std;
using namespace ROPI;

LP Rimpl_Strict::generate_robust()
{
  switch (unctype)
  {
    case UNC_FINITE:
      return generate_finite();
    case UNC_UNDEFINED:
      throw Error("In generate_robust: Uncertainty is undefined.");
    default:
      throw Error("In generate_robust: Unkown uncertainty.");
  }

}

Robustness_Method Rimpl_Strict::get_type()
{
  return ROB_STRICT;
}

LP Rimpl_Strict::generate_finite()
{
  list<Var>* lpvars = lp->get_vars();
  list<Con>* lpcons = lp->get_cons();
  LP rc;
  
  int nr_scenarios = finunc.scenarios.size();
  int nr_vars = lpvars->size();
  int nr_cons = lpcons->size();
  
  //don't forget the nominal case
  vector<Var> rcvars(nr_vars);
  vector<Con> rccons(nr_cons*(nr_scenarios+1));
  
  //variables
  for (list<Var>::iterator it=lpvars->begin(); it!=lpvars->end(); ++it)
    rcvars[it->id] = *it;
  
  //nominal case constraints
  int con_counter = 0;
  for (list<Con>::iterator it=lpcons->begin(); it!=lpcons->end(); ++it)
  {
    rccons[con_counter] = *it;
    stringstream conname;
    conname<<rccons[con_counter].name;
    conname<<"-SC-NOM";
    rccons[con_counter].name = conname.str();
    ++con_counter;
  }
  
  //scenario constraints
  for (int i=0; i<nr_scenarios; ++i)
  {
    for (list<Con>::iterator it=lpcons->begin(); it!=lpcons->end(); ++it)
    {
      rccons[con_counter] = *it;
      stringstream conname;
      conname<<rccons[con_counter].name;
      conname<<"-SC-"<<i;
      rccons[con_counter].name = conname.str();
      rccons[con_counter].id = it->id + (i+1)*nr_cons;
      //neue coeffs
      for (list<pair<int,double> >::iterator coeffit=rccons[con_counter].coeffs.begin(); coeffit!=rccons[con_counter].coeffs.end(); ++coeffit)
      {
	if (finunc.scenarios[i].coeffs.count(make_pair<Con,int>(*it,coeffit->first)) > 0)
	  coeffit->second = finunc.scenarios[i].coeffs[make_pair<Con,int>(*it,coeffit->first)];
      }
      if (finunc.scenarios[i].rhs.count(*it) > 0)
	rccons[con_counter].rhs = finunc.scenarios[i].rhs[*it];
      
      ++con_counter;
 
    }
  }

  //add to lp model
  for (unsigned int i=0; i<rcvars.size(); ++i)
    rc.add_var(rcvars[i]);
    
  for (unsigned int i=0; i<rccons.size(); ++i)
    rc.add_con(rccons[i]);
  
  rc.set_name(lp->get_name() + "-STRICT");
  
  return rc; 
}