// **************************************************
// This file is part of the ROPI Library.
// Written by Marc Goerigk
// Institut für Numerische und Angewandte Mathematik
// Georg-August Universität Göttingen
// **************************************************

#include "robustness.h"
#include "error.h"
#include "rimpl_recfeas.h"
#include "rimpl_strict.h"
#include "rimpl_light.h"
#include "rimpl_recopt.h"

using namespace std;
using namespace ROPI;

Robustness::Robustness(LP* _lp, Robustness_Method method)
{
  switch (method)
  {
    case ROB_STRICT:
      rimpl = new Rimpl_Strict(_lp);
      break;
    case ROB_LIGHT:
      rimpl = new Rimpl_Light(_lp);
      break;
    case ROB_RECOPT:
      rimpl = new Rimpl_Recopt(_lp);
      break;
    case ROB_RECFEAS:
      rimpl = new Rimpl_Recfeas(_lp);
      break;
    case ROB_UNDEFINED:
      throw Error("Unknown robustness concept.");
    default:
      throw Error("No robustness concept given.");
  }
}

void Robustness::set_uncertainty(FiniteUncertainty _unc)
{
  rimpl->set_uncertainty(_unc);
}

LP Robustness::generate_robust()
{
    return rimpl->generate_robust();
}

void Robustness::exclude_variable(int id)
{
  if (rimpl->get_type() == ROB_RECFEAS)
    (static_cast<Rimpl_Recfeas*>(rimpl))->exclude_variable(id);
}

void Robustness::set_options(RobustnessOptions _options)
{
  rimpl->set_options(_options);
}
